C ======================================================================
C SUBROUTINE COHESIVE
C
C Compute the erosion or deposition of cohesive sediment during a time
C step TIMEDR for a given shear velocity USTCWS.
C Return alos the average transport rate of suspended sediment SEDM.

      SUBROUTINE COHESIVE (CONC,BEDCHA,MAXBED,TIMEDR,USTCWS,U100,D,
     &RHOW,VISK,AULVA,EDRATE,ZS,SEDM,TCONC,TAOS)

      IMPLICIT NONE

C ************* CONSTANTS TRANSMITTED BY BLOCK *************
      INCLUDE "sed_param.h"   ! declaration of the common blocks

C ********************** INPUT VARIABLES *******************
      DOUBLE PRECISION CONC(NBCONC) !array of suspended sediment concentration (kg/m3)
                               ! for each Ws class,               user input
                               ! Ws of each class is in WSI(i) (common block common WSCLASS)
      INTEGER MAXBED           ! upper bound of first dimension of BEDCHA
      DOUBLE PRECISION BEDCHA(MAXBED,3) ! bed characteristics in 3 column table
                               ! (1) depth below sediment surface (m)
                               ! (2) critical erosion stress (Pa)
                               ! (3) dry bulk density (kg/m**3)          user input
      DOUBLE PRECISION TIMEDR  ! Duration of call to Sedtrans (s)         user input
      DOUBLE PRECISION USTCWS  ! Combined skin friction stress            FRICFRAC
      DOUBLE PRECISION U100    ! current speed at 1m above seabed (m/sec) FRICFRAC
      DOUBLE PRECISION D       ! water depth (m)                          user input
      DOUBLE PRECISION VISK    ! kinematic viscosity (m**2/s)            from main program
      DOUBLE PRECISION RHOW    ! density of fluid (water)  (kg/m**3)      user input
      DOUBLE PRECISION AULVA   ! fraction of bed area covered by the algae 'Ulva' (range 0-1) user input

C ********************** OUTPUT VARIABLES *******************
      DOUBLE PRECISION EDRATE  ! mean erosion/deposition rate (kg/m^2/s)
                               ! (positive: erosion, negative: deposition
      DOUBLE PRECISION ZS      ! Height change in bed surface (m)
                               ! (positive: erosion, negative: deposition)
      DOUBLE PRECISION SEDM    ! time-averaged net sediment transport as mass (kg/s/m)
      DOUBLE PRECISION TCONC   ! final total suspended sediment concentration (kg/m**3)
      DOUBLE PRECISION TAOS    ! solid transmitted stress due to Ulva (Pa)

C ********************** INTERMEDIATE VARIABLES *******************
      INTEGER NBED             ! number of rows in BEDCHA that are used
      DOUBLE PRECISION TAU0    ! effective skin friction shear stress (Pa)
      DOUBLE PRECISION TCONC1  ! total SSC (sum of CONC) before deposition (kg/m**3)
      DOUBLE PRECISION DMASS   ! deposited mass (kg)
      DOUBLE PRECISION EMASS   ! eroded mass (kg)
      DOUBLE PRECISION TAUCE   ! critical erosion stress at depth ZS (Pa)
      DOUBLE PRECISION MAXEMASS! Maximum erodable sediment mass to avoid problem with drag reduction

      INTEGER I              ! counter for loops

C **********************************************************************
C Initialisation of some variables
      ZS=0d0
      EMASS=0d0		          ! Eroded mass
      DMASS=0d0                ! Deposited mass
      TCONC1=0.D0

C **********************************************************************
C Find the number of rows that are used (NBED). This is indicated by z=0 in
C the row after the last used row.

      DO 10, I=2,MAXBED
         IF (BEDCHA(I,1).EQ.0.0) GOTO 20
10    CONTINUE
20    NBED=I-1

C **********************************************************************
C Compute the total SSC before deposition and erosion

      DO 210, I=1,NBCONC
         TCONC1=TCONC1+CONC(I)
210   CONTINUE

C **********************************************************************
C Compute TAU0, the effective skin friction stress acting on the bed
C drag reduction and solid transmitted stress due to ulva

      TAU0=RHOW*USTCWS**2

      CALL DRAGRED(TAU0,TCONC1,D,MAXEMASS)

      CALL SOLULVA(TAU0,AULVA,TAOS)

C **********************************************************************
C EROSION FIRST PART
C Test if effective bed shear stress (TAU0) is larger or equal to
C surface critical erosion stress (BEDCHA(1,2)

      IF ( TAU0.GT.BEDCHA(1,2) .OR. ( TAU0.EQ.BEDCHA(1,2).AND.
     &   ( TAU0.GE.BEDCHA(2,2) .OR. NBED.EQ.1) ) ) THEN

         CALL EROS1(NBED,MAXBED,BEDCHA,TAU0,MAXEMASS,EMASS,ZS,
     @TIMEDR,TAUCE)

         CALL BEDERO(NBED,MAXBED,BEDCHA,ZS)

      ENDIF    ! end of erosion, 1st part

C **********************************************************************
C DEPOSITION
C Compute deposition only if there are suspended sediments

      IF (TCONC1.EQ.0d0) THEN
         TCONC=0.D0
      ELSE
         CALL DEPOS(D,TIMEDR,CONC,TAU0,RHOW,TCONC1,TCONC,VISK,DMASS)

         CALL BEDDEP(NBED,MAXBED,BEDCHA,TAU0,DMASS,ZS)
      ENDIF

C **********************************************************************
C EROSION SECOND PART
C Put eroded sediment in suspension

      IF (EMASS.GT.0.) THEN
         CALL EROS2(TAUCE,D,RHOW,TAU0,CONC,EMASS,TCONC)
      ENDIF

C **********************************************************************
C FINAL EROSION/DEPOSITION OPERATION
C Compute the deposition rate

      EDRATE=(EMASS-DMASS)/TIMEDR

C **********************************************************************
C Sediment transport is computed from the mean SSC

      SEDM=0.5d0*(TCONC1+TCONC) * D * U100

C **********************************************************************
C Apply bed-compaction for the duration TIMEDR
      IF (DOCOMPACT.NE.0) CALL COMPACT(BEDCHA,MAXBED,TIMEDR,ZS)

      END   ! end of subroutine COHESIVE

C ======================================================================
C ======================================================================
C SUBROUTINE EROS1
C EROSION FIRST PART : test if erosion
C                      compute erosion rate and eroded depth, and
C                      check that TAUCE(ZS)<=TAU0
C                      compute eroded mass EMASS, which is used later

      SUBROUTINE EROS1(NBED,MAXBED,BEDCHA,TAU0,MAXEMASS,EMASS,ZS,
     @TIMEDR,TAUCE)

      IMPLICIT NONE

      INCLUDE "sed_param.h"    ! declaration of the common blocks

C **** INPUT VARIABLES ****
      INTEGER NBED             ! number of rows in BEDCHA that are used
      INTEGER MAXBED           ! upper bound of first dimension of BEDCHA
      DOUBLE PRECISION BEDCHA(MAXBED,3) ! bed characteristics in 3 column table
                               ! (1) depth below sediment surface (m)
                               ! (2) critical erosion stress (Pa)
                               ! (3) dry bulk density (kg/m**3)          user input
      DOUBLE PRECISION TAU0    ! effective skin friction shear stress (Pa)
      DOUBLE PRECISION MAXEMASS! Maximum erodable sediment mass to avoid problem with drag reduction
      DOUBLE PRECISION TIMEDR  ! Duration of call to Sedtrans (s)         user input
C **** OUTPUT VARIABLES ****
      DOUBLE PRECISION EMASS   ! eroded mass (kg)
      DOUBLE PRECISION RHOS    ! dry bulk density at depth ZS (kg/m**3)
      DOUBLE PRECISION TAUCE   ! critical erosion stress at depth ZS (Pa)
      DOUBLE PRECISION ZS      ! Height change in bed surface (m)
                               !(positive: erosion, negative: deposition)
C **** INTERMEDIATE VARIABLES ****
      DOUBLE PRECISION DZ      ! Height of deposited mud layer (m) / eroded depth in layer IB
      DOUBLE PRECISION T1      ! remaining time during erosion step
      DOUBLE PRECISION M1      ! eroded mass in this layer
      DOUBLE PRECISION RE      ! erosion rate (kg/m**2/s)
      DOUBLE PRECISION RHOA,RHOB,HLAYER,MLAYER,TAUCES ! for erosion steps
      INTEGER I                ! counter for loops
      INTEGER IB               ! counter for loops in BEDCHA

C Statement function which interpolate linearly the value Y of (X,Y),
C which located on the line between (X1,Y1) and (X2,Y2)
      DOUBLE PRECISION LIN,X1,X2,Y1,Y2,X
      LIN(X1,X2,Y1,Y2,X)=Y1 + (X-X1)/(X2-X1)*(Y2-Y1)

C Find the erosion depth ZS and also TAUCE and RHOS at this depth
C Each layer is processed successively until the duration TIMEDR is finished
      T1=TIMEDR ! time remaining for erosion
      DO 110, IB=1,NBED
C Density variation in this layer RHO = RHOA * dz + RHOB
         RHOB=BEDCHA(IB,3)
         IF(IB.EQ.NBED) THEN
            RHOA=0d0
            HLAYER=1000d0
         ELSE
            HLAYER=(BEDCHA(IB+1,1)-BEDCHA(IB,1))
            RHOA=(BEDCHA(IB+1,3)-BEDCHA(IB,3)) / HLAYER
         ENDIF
         MLAYER=(RHOB+RHOA*HLAYER/2d0)*HLAYER   ! total mass of layer
C Do 2 iterations do define the eroded depth, for the firt RE is calculated
C with TAUCES = Tce(top), for the second TAUCES = (Tce(top)+Tce(dz))/2
         TAUCES=BEDCHA(IB,2) ! for first iteration
         DO 100, I=1,2
            RE=E0*EXP(RKERO*(TAU0-TAUCES)**0.5d0) ! erosion rate
            M1=RE*T1    ! eroded mass
            M1=MIN(M1,MAXEMASS-EMASS)
            IF(M1.GE.MLAYER) THEN
               DZ=HLAYER    ! erosion depth in this layer
               TAUCE=BEDCHA(MIN(IB+1,NBED),2)
            ELSE
               IF(RHOA.EQ.0) THEN
                  DZ=M1/RHOB
               ELSE
                  DZ=(-RHOB+SQRT(RHOB**2+2d0*RHOA*M1))/RHOA
               ENDIF
               IF(IB.EQ.NBED) THEN
                  TAUCE=BEDCHA(IB,2)
               ELSE
                 TAUCE=LIN(0d0,HLAYER,BEDCHA(IB,2),BEDCHA(IB+1,2),DZ)
               ENDIF
            ENDIF
            IF(TAU0.LT.TAUCE)THEN ! check if erosion went to deep (into too strong sediments
               ZS=LIN(BEDCHA(IB,2),BEDCHA(IB+1,2),
     &               BEDCHA(IB,1),BEDCHA(IB+1,1),TAU0)
               TAUCE=TAU0
               RHOS=LIN(BEDCHA(IB,1),BEDCHA(IB+1,1),
     &              BEDCHA(IB,3),BEDCHA(IB+1,3),ZS)
               GOTO 120  ! out of loop DO 100
            ENDIF
            IF(I.EQ.1) THEN
               TAUCES=(TAUCES+TAUCE)/2d0 ! for second iteration
            ELSE
               IF(M1.LE.MLAYER)THEN ! the layer is not eroded completely
                  RHOS=DZ*RHOA+RHOB
                  ZS=DZ+BEDCHA(IB,1)
                  GOTO 120  ! out of loop DO 100
               ENDIF
               T1=T1-(MLAYER/RE) ! substract from T1 the time taken to erode this layer
            ENDIF
100      CONTINUE
         EMASS=EMASS+M1  ! first approximation for drag-reduction limitation
110   CONTINUE  ! this loop will always be exited by a GOTO 120

120   IF(IB.LT.NBED) IB=IB+1
C At this stage, IB can have following values
C    if ZS > BEDCHA(NBED,1)               IB=NBED   (also if NBED=1)
C    if ZS = BEDCHA(n,1)                  IB=n
C    if BEDCHA(n-1,1) < ZS < BEDCHA(n,1)  IB=n

C Compute the eroded mass, EMASS
      EMASS=0d0 ! EMASS was used above for temporary storage
      IF(ZS.GT.BEDCHA(NBED,1) .OR. ZS.EQ.BEDCHA(IB,1)) THEN
         DO 140, I=1,IB-1
            EMASS=EMASS + (BEDCHA(I+1,1)-BEDCHA(I,1))*
     &         (BEDCHA(I,3)+BEDCHA(I+1,3))*0.5d0   ! dZ * (RHOS1+RHOS2)/2
140      CONTINUE
         EMASS=EMASS + (ZS-BEDCHA(IB,1))*BEDCHA(IB,3) ! dZ * RHOS
      ELSE
         DO 150, I=1,IB-2
            EMASS=EMASS + (BEDCHA(I+1,1)-BEDCHA(I,1))*
     &         (BEDCHA(I,3)+BEDCHA(I+1,3))*0.5d0    ! dZ * (RHOS1+RHOS2)/2
150      CONTINUE
         EMASS=EMASS+(ZS-BEDCHA(IB-1,1))*(BEDCHA(IB-1,3)+RHOS)*0.5d0
      ENDIF

      END

C ======================================================================
C SUBROUTINE BEDERO(MAXBED,BEDCHA,ZS)
C Compute the bed characteristics after erosion (truncate the bed at depth ZS)

      SUBROUTINE BEDERO(NBED,MAXBED,BEDCHA,ZS)

      IMPLICIT NONE

      INTEGER NBED             ! number of rows in BEDCHA that are used
      INTEGER MAXBED           ! upper bound of first dimension of BEDCHA
      DOUBLE PRECISION BEDCHA(MAXBED,3) ! bed characteristics in 3 column table
                               ! (1) depth below sediment surface (m)
                               ! (2) critical erosion stress (Pa)
                               ! (3) dry bulk density (kg/m**3)
      DOUBLE PRECISION ZS      ! Height change in bed surface (m)
      DOUBLE PRECISION TAUCE   ! critical erosion stress at depth ZS (Pa)
      DOUBLE PRECISION RHOS    ! dry bulk density at depth ZS (kg/m**3)
      INTEGER MOVEZ            ! nb of row that are reshuffled after erosion
      INTEGER I                ! counter for loops
      INTEGER IB               ! first row in BEDCHA strictly below ZS
C Statement function interpolating linearly the value Y of (X,Y),
C which is located on the line between (X1,Y1) and (X2,Y2)
      DOUBLE PRECISION LIN,X1,X2,Y1,Y2,X
      LIN(X1,X2,Y1,Y2,X)=Y1 + (X-X1)/(X2-X1)*(Y2-Y1)

      IF (NBED.GT.1) THEN      ! nothing has to be done if NBED=1
         IF (ZS.GE.BEDCHA(NBED,1)) THEN
C The bed is eroded below the deepest point of BEDCHA
            TAUCE=BEDCHA(NBED,2)
            RHOS=BEDCHA(NBED,3)
            MOVEZ=NBED-1
            IB=NBED+1
         ELSE
C Find next point below erosion depth, and interpolate TAUCE and RHOS
            DO 20,IB=2,NBED
               IF (ZS.LT.BEDCHA(IB,1)) GOTO 30
20          CONTINUE
30          TAUCE=LIN(BEDCHA(IB-1,1), BEDCHA(IB,1),
     &                BEDCHA(IB-1,2), BEDCHA(IB,2), ZS)
            RHOS= LIN(BEDCHA(IB-1,1), BEDCHA(IB,1),
     &                BEDCHA(IB-1,3), BEDCHA(IB,3), ZS)
            MOVEZ=IB-2
         ENDIF
C Modify BEDCHA
         BEDCHA(1,2)=TAUCE
         BEDCHA(1,3)=RHOS
         DO 170, I=IB,NBED
            BEDCHA(I-MOVEZ,1)=BEDCHA(I,1)-ZS
            BEDCHA(I-MOVEZ,2)=BEDCHA(I,2)
            BEDCHA(I-MOVEZ,3)=BEDCHA(I,3)
170      CONTINUE
C  set rows after last used row to zero
         DO 180, I=NBED-MOVEZ+1,NBED
            BEDCHA(I,1)=0d0
            BEDCHA(I,2)=0d0
            BEDCHA(I,3)=0d0
180      CONTINUE
         NBED=NBED-MOVEZ
      ENDIF

      END

C ======================================================================
C SUBROUTINE DEPOS
C DEPOSITION: - deposit only if SSC>0
C             - loop trough each Ws class
C             - compute TAUCD (include floccultion effects)

      SUBROUTINE DEPOS(D,TIMEDR,CONC,TAU0,RHOW,TCONC1,TCONC2,
     @VISK,DMASS)

      IMPLICIT NONE

      INCLUDE "sed_param.h"    ! declaration of the common blocks

C **** INPUT VARIABLES ****
      DOUBLE PRECISION D       ! water depth (m)                          user input
      DOUBLE PRECISION TIMEDR  ! Duration of call to Sedtrans (s)         user input
      DOUBLE PRECISION TAU0    ! effective skin friction shear stress (Pa)
      DOUBLE PRECISION RHOW    ! density of fluid (water)  (kg/m**3)      user input
      DOUBLE PRECISION TCONC1  ! total SSC (sum of CONC) before deposition (kg/m**3)
      DOUBLE PRECISION VISK    ! kinematic viscosity (m**2/s)            from main program
C **** INPUT and OUTPUT VARIABLES ****
      DOUBLE PRECISION CONC(NBCONC) !array of suspended sediment concentration (kg/m3)
                               ! for each Ws class,               user input
                               ! Ws of each class is in WSI(i) (common block common WSCLASS)
C **** OUTPUT VARIABLES ****
      DOUBLE PRECISION TCONC2  ! total SSC (sum of CONC) after deposition (kg/m**3)
      DOUBLE PRECISION DMASS   ! deposited mass (kg)
C **** INTERMEDIATE VARIABLES ****
      DOUBLE PRECISION TAUCD   ! critical deposition stress for the considered floc class (Pa)
      DOUBLE PRECISION WS      ! settling velocity of the considered floc class (m/s)
C for flocculation calculation
      DOUBLE PRECISION WSFLOC  ! Ws computed for flocculation
      DOUBLE PRECISION DE,CF   ! intermediate value for flocculation equation
      DOUBLE PRECISION RHOFLOC ! density of flocs
      DOUBLE PRECISION CVOL    ! volume concentration
      DOUBLE PRECISION MEANWS  ! ~mean log(Ws)
      INTEGER I                ! counter for loops

C Computed median Ws of flocculated particles (WSFLOC) according to Whitehouse et al. 1999
C An alternative would be to use Andy Manning's formula which include also turbulence
      IF(TCONC1.GT.CLIM1) THEN
         IF(TCONC1.LE.CLIM2) THEN
            WSFLOC=KFLOC*(TCONC1**MFLOC)
         ELSEIF(TCONC1.LE.50) THEN
            RHOFLOC=RHOW+0.03d0*(RHOCLAY-RHOW)
            CVOL=TCONC1/RHOCLAY
            DE=SQRT(19.8d0*RHOW*VISK*RHOCLAY**MFLOC*KFLOC/
     &         (9.81d0*(RHOFLOC-RHOW))) * CVOL**(MFLOC/2d0)
            CF=(RHOCLAY-RHOW)*CVOL/(RHOFLOC-RHOW)
            WSFLOC=VISK/DE*((10.36d0**2+1.049d0*(1-CF)**4.7d0*
     &         DE**3 * 9.81d0*(RHOFLOC-RHOW)/(RHOW*VISK**2) )**0.5d0
     &         -10.36d0)
         ELSEIF(TCONC1.LE.82) THEN
            WSFLOC=0.00462d0*(1d0-0.01d0*TCONC1)**3.54d0
         ELSE
            WSFLOC=1d-5
         ENDIF
C compute the mean log(Ws)
         MEANWS=0
         DO 212,I=1,NBCONC
            MEANWS=MEANWS+LOG10(WSI(I))*CONC(I)
212      CONTINUE
         MEANWS=10**(MEANWS/TCONC1)

      ENDIF

C LOOP FOR EACH SUSPENDED SEDIMENT CLASS
      DO 220 I=NBCONC,1,-1
         IF (CONC(I).GT.0.D0) THEN ! SKIP CLASS IF EMPTY
            WS=WSI(I) ! compute free settling velocity for this class
            IF(TCONC1.GT.CLIM1) THEN ! correct for flocculation
               WS=MAX(WS,WSFLOC* SQRT(WS/MEANWS) )
            ENDIF
            TAUCD=CTAUDEP*RHOW*0.64D0*WS**2 ! compute the critical stress for deposition

C Check if effective skin friction stress below TAUCD
            IF (TAU0.GE.TAUCD) THEN
C if this class do not settle, the finer class will also not  --> exit Ws loop 220
               GOTO 230
            ELSEIF (CONC(I).GT.0) THEN
C COMPUTE THE CONCENTRATION REMAINING IN SUSPENSION AFTER DEPOSITION DURING TIME TIMEDR
               CONC(I)=CONC(I) *
     &                 EXP(-WS*(1D0-TAU0/TAUCD)*(1d0-PRS)*TIMEDR/D)
C IF CONCENTRATION IN ONE CLASS FALL BELOW 0.00001 KG/M**3, DEPOSIT EVERYTHING
               IF (CONC(I) .LE. 0.00001D0) THEN
                  CONC(I)=0.D0
               ENDIF
            ENDIF
         ENDIF
220   CONTINUE

C COMPUTE THE TOTAL SSC AFTER THE DEPOSITION (but before erosion)
230   TCONC2=0.d0
      DO 240 I=1,NBCONC
        TCONC2=TCONC2+CONC(I)
240   CONTINUE
C Compute the total deposited mass (positive value)
      DMASS=(TCONC1-TCONC2)*D

      END

C ======================================================================
C SUBROUTINE BEDDEP
C Compute the bed characteristics after deposition

      SUBROUTINE BEDDEP(NBED,MAXBED,BEDCHA,TAU0,DMASS,ZS)

      IMPLICIT NONE

      INCLUDE "sed_param.h"    ! declaration of the common blocks

C **** INPUT VARIABLES ****
      INTEGER NBED             ! number of rows in BEDCHA that are used
      INTEGER MAXBED           ! upper bound of first dimension of BEDCHA
      DOUBLE PRECISION TAU0    ! effective skin friction shear stress (Pa)
      DOUBLE PRECISION DMASS   ! deposited mass (kg)
C **** INPUT and OUTPUT VARIABLES ****
      DOUBLE PRECISION BEDCHA(MAXBED,3) ! bed characteristics in 3 column table
                               ! (1) depth below sediment surface (m)
                               ! (2) critical erosion stress (Pa)
                               ! (3) dry bulk density (kg/m**3)          user input
C **** OUTPUT VARIABLES ****
      DOUBLE PRECISION ZS      ! Height change in bed surface (m)
C **** INTERMEDIATE VARIABLES ****
      DOUBLE PRECISION DZ      ! Height of deposited mud layer (m) / eroded depth in layer IB
      INTEGER IB               ! counter for loops in BEDCHA

      IF(DMASS.GT.0) THEN
C ADD THE DEPOSITED SEDIMENT TO THE BED
         IF(BEDCHA(2,1).LT.0.01. AND. ABS(BEDCHA(1,3)-RHOMUD).LT.1.
     &   .AND. ABS(BEDCHA(2,3)-RHOMUD).LT.2. .AND. NBED.GT.1) THEN
C If the surface is already a fluid mud layer (thickness<1cm, density close to RHOMUD)
C add sediment to the top layer
C   (DZ+z2)*(RHOMUD+r2)/2 = DMASS + z2*(r1+r2)/2
            DZ=(2*DMASS + BEDCHA(2,1)*(BEDCHA(1,3)+BEDCHA(2,3))) /
     &            (RHOMUD+BEDCHA(2,3)) - BEDCHA(2,1)
            DO 250, IB=2,NBED
               BEDCHA(IB,1)=BEDCHA(IB,1)+DZ
250         CONTINUE
         ELSE
C otherwise add a new layer to the bed
C   DZ*(RHOMUD+r1)/2 = DMASS      DZ has a POSITIVE value for deposition
            DZ=DMASS*2/(RHOMUD+BEDCHA(1,3))

            IF(NBED.EQ.MAXBED) THEN
               CALL REMOVEBED(BEDCHA,MAXBED,-1d0)
               NBED=NBED-1
            ENDIF
C Shift all element of BEDCHA one row down
            DO 260, IB=NBED,1,-1
               BEDCHA(IB+1,1)=BEDCHA(IB,1)+DZ
               BEDCHA(IB+1,2)=BEDCHA(IB,2)
               BEDCHA(IB+1,3)=BEDCHA(IB,3)
260         CONTINUE
            NBED=NBED+1
            IF (NBED.LT.MAXBED) BEDCHA(NBED+1,1)=0
         ENDIF

C Compute a erosion threshold slighly higher than current TAU0, but not
C higher than the previous bed surface.
         BEDCHA(1,2)=MIN(TAU0*1.1d0,BEDCHA(1,2))
C erosion threshold not lower than a minimal value, using the same formula
C than in the subroutine COMPACT
         BEDCHA(1,2)=MAX(BEDCHA(1,2),TEROA*(RHOMUD**TEROB))

         BEDCHA(1,3)=RHOMUD
         ZS=ZS-DZ          !  ZS has a NEGATIVE value for deposition

      ENDIF

      END

C ======================================================================
C SUBROUTINE EROS2
C EROSION SECOND PART : put eroded sediment in suspension

      SUBROUTINE EROS2(TAUCE,D,RHOW,TAU0,CONC,EMASS,TCONC2)

      IMPLICIT NONE

      INCLUDE "sed_param.h"    ! declaration of the common blocks

C **** INPUT VARIABLES ****
      DOUBLE PRECISION TAUCE   ! critical erosion stress at depth ZS (Pa)
      DOUBLE PRECISION D       !water depth (m)                          user input
      DOUBLE PRECISION RHOW    ! density of fluid (water)  (kg/m**3)      user input
      DOUBLE PRECISION TAU0    ! effective skin friction shear stress (Pa)
      DOUBLE PRECISION EMASS   ! eroded mass (kg)
C **** OUTPUT VARIABLES ****
      DOUBLE PRECISION CONC(NBCONC) !array of suspended sediment concentration (kg/m3)
                               ! for each Ws class,               user input
                               ! Ws of each class is in WSI(i) (common block common WSCLASS)
      DOUBLE PRECISION TCONC2  ! total SSC (sum of CONC) after deposition (kg/m**3)
C **** INTERMEDIATE VARIABLES ****
      DOUBLE PRECISION WS      ! settling velocity of the considered floc class (m/s)
      INTEGER WSERODED         ! largest Ws class put in suspension + 1
      INTEGER I,J              ! counter for loops

C Determine Ws of suspended sediment, WSERODED is median+1 Ws index
      DO 310, WSERODED=WSCLAY+1,NBCONC
C Find the first Ws class that cannot be lifted
         WS=WSI(MIN(WSERODED+NDISTR-MEDDISTR,NBCONC)) ! Ws of coarsest of distribution
         IF (RHOW*0.64D0*WS**2.GT.TAU0) GOTO 320
C Find the first Ws class that is disrupted
         WS=WSI(WSERODED)  ! Ws of median of distribution
         IF (WS .GE. TAUCE/(TAU0**0.5d0) * CDISRUPT) GOTO 320
310   CONTINUE
      WSERODED=NBCONC+1

320   CONTINUE
CCC      WSERODED=WSERODED-3
      DO 330,I=1,NDISTR
         J=WSERODED-MEDDISTR+I-1
         J=MAX(MIN(J,NBCONC),1)  ! put J in the range 1:NBCONC
         CONC(J)=CONC(J)+EMASS/D*DISTR(I)
330   CONTINUE

C compute the total SSC after the erosion (to be used for the sediment transport)
      TCONC2=0
      DO 390 I=1,NBCONC
         TCONC2=TCONC2+CONC(I)
390   CONTINUE

      END

C ======================================================================
C SUBROUTINE DRAGRED
C Calculate the drag reduction as function of SSC. Calculate the
C maximum erodable mass so that TAU0 is halfed because of drag reduction

      SUBROUTINE DRAGRED(TAU0,TCONC1,D,MAXEMASS)

      IMPLICIT NONE

      INCLUDE "sed_param.h"	! declaration of the common blocks

      DOUBLE PRECISION TCONC1	! total SSC (sum of CONC) before deposition (kg/m**3)
      DOUBLE PRECISION MAXEMASS	! Maximum erodable sediment mass to avoid problem with drag reduction
      DOUBLE PRECISION TAU0	! effective skin friction shear stress (Pa)
      DOUBLE PRECISION D	! water depth (m)                          user input

      IF(CDRAGRED.LT.0)THEN
         TAU0=TAU0*EXP(CDRAGRED*TCONC1)
         MAXEMASS=D*(-0.693147)/CDRAGRED   ! -0.693147 = LOG(0.5)
      ELSEIF(CDRAGRED.EQ.0)THEN
         MAXEMASS=1d10		! no limitations if no drag reduction
      ENDIF

      END

C ======================================================================
C SUBROUTINE SOLULVA
C CALCULATE SOLID TRANSMITTED STRESS DUE TO ULVA: TAOS
C Uses P. Cozette's formulae (MSc thesis, 2000, SOC)

      SUBROUTINE SOLULVA(TAU0,AULVA,TAOS)

      IMPLICIT NONE

      INCLUDE "sed_param.h"	! declaration of the common blocks

      DOUBLE PRECISION TAU0	! effective skin friction shear stress (Pa)
      DOUBLE PRECISION AULVA	! fraction of bed area covered by the algae 'Ulva' (range 0-1)
      DOUBLE PRECISION TAOS	! solid transmitted stress due to Ulva (Pa)

      IF (AULVA.LE.0.0) THEN	! Compute nothing if there is no Ulva
         TAOS=0.0d0
      ELSE
         IF(TAU0.LE.TMULVA .OR. TAU0.GE.TRULVA) THEN
            TAOS=0.0d0        ! either no motion of Ulva, or full resuspension
         ELSEIF(TAU0.LE.0.8d0*TRULVA) THEN
            TAOS=AULVA*CSULVA*(TAU0-TMULVA) ! normal case
         ELSE
            TAOS=AULVA*CSULVA*(TRULVA-TAU0)/(0.2d0*TRULVA)*
     &       (0.8d0*TRULVA-TMULVA) ! partial resusponsion
         ENDIF
C CALCULATE NEW TOTAL SHEAR STRESS
         TAU0=TAU0+TAOS
      ENDIF
C Save TAU0 to TAU0EFF, which is return in common block
      TAU0EFF=TAU0

      END

C ======================================================================
C SUBROUTINE COMPACT
C
C Compute the compaction and the consolidation of the bed
C for a give time duration (TIMEDR).
C If necessary or advantageous, simplify the bed structure.
C BEDCHA is input and output argument, containing the bed characteristics.

      SUBROUTINE COMPACT(BEDCHA,MAXBED,TIMEDR,ZS)
      IMPLICIT NONE
C input/output variables
      INTEGER MAXBED          ! number of rows in variable BEDCHA
      DOUBLE PRECISION BEDCHA(MAXBED,3) ! bed characteristics in 3 column table
                              ! (1) DEPTH BELOW SEDIMENT SURFACE (m)
                              ! (2) CRITICAL EROSION STRESS (Pa)
                              ! (3) DENSITY (kg/m**3)
      DOUBLE PRECISION TIMEDR ! Duration of call to Sedtrans)
      DOUBLE PRECISION ZS     ! change in height of bed surface (m)
                              ! positive when bed compacted

C ************* CONSTANTS TRANSMITTED BY COMMON BLOCK *************
      INCLUDE "sed_param.h"  ! declaration of the common blocks


C intermediate variables
      INTEGER HIMAXBED
      PARAMETER (HIMAXBED=50) ! Highest possible value for MAXBED
      DOUBLE PRECISION OLDRHO(HIMAXBED)! old densities (column 3 of BEDCHA)
      DOUBLE PRECISION DZOLD(HIMAXBED) ! new height a bed layer
      DOUBLE PRECISION ABOVE(HIMAXBED) ! mass of sediment above (kg/m**2)
      DOUBLE PRECISION DZNEW  ! new height of a bed layer
      DOUBLE PRECISION FINAL  ! final density for a given over-weight
      DOUBLE PRECISION TAUCE  ! new computed erosion threshold (Pa)
      INTEGER NBED            ! number of rows in BEDCHA that are used
      INTEGER IB              ! counter for loops in BEDCHA

C Find the number of rows that are used (NBED). This is indicated by z=0 in
C row after the last used row.
      ABOVE(1)=0 ! mass above the surface is 0
      DO 10, IB=2,MAXBED
         IF (BEDCHA(IB,1).EQ.0.0) GOTO 20
         ! compute the mass above depth Z : ABOVE(IB-1) + dZ * mean RHOS
         ABOVE(IB)=ABOVE(IB-1) + (BEDCHA(IB,1)-BEDCHA(IB-1,1))*
     &               0.5d0*(BEDCHA(IB,3)+BEDCHA(IB-1,3))
         DZOLD(IB)=BEDCHA(IB,1)-BEDCHA(IB-1,1)
10    CONTINUE
20    NBED=IB-1

C Compute the new bed densities = ACTUAL COMPACTION
      DO 110, IB=1,NBED
         OLDRHO(IB)=BEDCHA(IB,3)
C Compute the final density for this overlaying mass
C     FINAL(I) = A -  B * EXP(-C*ABOVE(I)) - D * EXP(-E*ABOVE(I))
C     A : final deep density
C     A-B-D : final surface density
C     C and E : define the shape (in conjunction with B and C)
         FINAL=DPROFA-DPROFB*EXP(-DPROFC*ABOVE(IB))
     &            -DPROFD*EXP(-DPROFE*ABOVE(IB))
         IF (FINAL.GT.BEDCHA(IB,3)) THEN
                       ! do nothing if already more consolidated than final

C exonential decrease of difference between density and final density
C (RHO2-RHOfinal) = (RHO1-RHOfinal) * exp(-a*dt)
            BEDCHA(IB,3)=FINAL +
     &         (BEDCHA(IB,3)-FINAL) * exp(-CONSOA*TIMEDR)
         ENDIF
C Compute the new Erosion threshold TAUCE
C The same formula is used for the freshly deposited sediment in BEDDEP
C If the computed TAUCE is smaller than the previous one, keep the previous one

         TAUCE=TEROA*(BEDCHA(IB,3)**TEROB)*
     &         (1+TEROC*(1-EXP(TEROD*ABOVE(IB))))
         BEDCHA(IB,2)=MAX(BEDCHA(IB,2),TAUCE)

110   CONTINUE

C Adapt the depth Z to the new densities by conserving the sediment mass
      DO 200, IB=2,NBED
         DZNEW=DZOLD(IB)*
     &      ((OLDRHO(IB)+OLDRHO(IB-1)) / (BEDCHA(IB,3)+BEDCHA(IB-1,3)))
         ZS=ZS+(DZOLD(IB)-DZNEW) ! ZS positif for consolidation/compaction
         BEDCHA(IB,1)=BEDCHA(IB-1,1)+DZNEW
200   CONTINUE

C ??????????????????????????????????????????????????????????????????????
C Reorganize BEDCHA if too much rows are used or the difference between
C the rows are too small


      END   ! end of subroutine COMPACT



C ======================================================================
C ======================================================================
C FUNCTION CHECKBEDCHA
C
C Check if the array BEDCHA follows the rules and contains plausible values.
C If found correct, returns .TRUE., otherwise returns .FALSE.

      LOGICAL FUNCTION CHECKBEDCHA(BEDCHA,MAXBED)
      IMPLICIT NONE
      INTEGER MAXBED          ! upper bound of first dimension of BEDCHA
      DOUBLE PRECISION BEDCHA(MAXBED,3) ! bed characteristics in 3 column table
                              ! (1) depth below sediment surface (m)
                              ! (2) critical erosion stress (Pa)
                              ! (3) dry bulk density (kg/m**3)          user input
      INTEGER HIMAXBED
      PARAMETER (HIMAXBED=50) ! Highest possible value for MAXBED
      INTEGER NBED, I

      IF (MAXBED.LT.3 .OR. MAXBED.GT.HIMAXBED) GOTO 90
         ! MAXBED must be in the range 3-50

      IF (BEDCHA(1,1).NE.0.0) GOTO 90  ! First entry must be the sediment surface

C Find the number of rows that are used (NBED)
      DO 10, I=2,MAXBED
         IF (BEDCHA(I,1).EQ.0.0) GOTO 20 ! A zero depth indicate the last entry
         IF (BEDCHA(I,1).LT.BEDCHA(I-1,1)) GOTO 90 ! Z must increase monotonously
10    CONTINUE
20    NBED=I-1

C Check that the values of TAUCE and RHOS are plausible
C   0 < TAUCE   and    0 < RHOS < 4000 kg/m**3
      DO 30, I=1,NBED
         IF (BEDCHA(I,2).LE.0 .OR. BEDCHA(I,3).LE.0 .OR.
     &      BEDCHA(I,3).GE.4000) GOTO 90
30    CONTINUE

C Check that all z after NBED are set to 0
      DO 40, I=NBED+1,MAXBED
         IF (BEDCHA(I,1).NE.0) GOTO 90
40    CONTINUE

      CHECKBEDCHA=.TRUE.    ! Found no error, return true
      RETURN

90    CHECKBEDCHA=.FALSE.   ! Found a error, return false
      END   ! end of function CHECKBEDCHA



C ======================================================================
C ======================================================================
C SUBROUTINE BEDINI
C
C Compute the initial bed characterics BEDCHA for a given surface density
C or critical erosion stress RHOSI
C Depending of the input range of RHOSI, RHOSI has different meaning:
C    0-50 : critical erosion stress (Pa)
C    >50  : dry bulk density of the surface (kg/m**3)

      SUBROUTINE BEDINI(BEDCHA,MAXBED,RHOSI)
      IMPLICIT NONE
      INCLUDE "sed_param.h"  ! declaration of the common blocks
C input variables
      INTEGER MAXBED         ! number of rows in variable BEDCHA
      DOUBLE PRECISION RHOSI ! at input, wet/dry density or TAUCE
C output variable
      DOUBLE PRECISION BEDCHA(MAXBED,3) ! bed characteristics in 3 column table
                              ! (1) DEPTH BELOW SEDIMENT SURFACE (m)
                              ! (2) CRITICAL EROSION STRESS (Pa)
                              ! (3) DENSITY (kg/m**3)

C intermediate variables
      DOUBLE PRECISION RHOSS  ! dry bulk density of surface (computed from RHOSI)
      INTEGER NBED            ! number of rows in BEDCHA that are used
      INTEGER IB              ! counter for loops in BEDCHA
      DOUBLE PRECISION A,B,C,D,E ! constant for density profile
      DOUBLE PRECISION RHOST  ! ideal surface density according constant DPROFA-E
      DOUBLE PRECISION M1,M2,MMID,ABOVE,DZ,DZEST ! for numerical iteration of densities
      INTEGER N

      IF (RHOSI.LE.50) THEN
C convert critical erosion stress to dry bulk density
C TAUCE=A*(RHOdry**B)
         RHOSS=(RHOSI/TEROA)**(1d0/TEROB)
      ELSE
         RHOSS=RHOSI
      ENDIF

C Define the constant for the density profile
      RHOST=DPROFA-DPROFB-DPROFD
      IF(RHOSS.LE.RHOST) THEN
         A=DPROFA
         B=DPROFB+(RHOST-RHOSS)*DPROFB/(DPROFB+DPROFD)
         C=DPROFC*B/DPROFB
         IF(DPROFD.NE.0)THEN
            D=DPROFD+(RHOST-RHOSS)*DPROFD/(DPROFB+DPROFD)
            E=DPROFE*D/DPROFD
         ELSE
            D=0
            E=0
         ENDIF
      ELSE
         IF(RHOSS.LT.1800) THEN
            A=(RHOSS-RHOST)*(1800d0-DPROFA)/(1800d0-RHOST)+DPROFA
            IF(A.GT.1800) A=1800d0
            B=(A-RHOSS)*DPROFB/(DPROFB+DPROFD)
            C=DPROFC*B/DPROFB
            IF(DPROFD.NE.0)THEN
               D=(A-RHOSS)*DPROFD/(DPROFB+DPROFD)
               E=DPROFE*D/DPROFD
            ELSE
               D=0
               E=0
            ENDIF
         ELSE
            A=RHOSS
            B=0
            C=0
            D=0
            E=0
         ENDIF
      ENDIF

C set the depth depending of the size of BEDCHA
      BEDCHA(1,1)=0
      IF (MAXBED.EQ.3) THEN
         NBED=2
         BEDCHA(2,1)=3d-3
      ELSEIF (MAXBED.GE.4 .AND. MAXBED.LE.5) THEN
         NBED=3
         BEDCHA(2,1)=2d-3
         BEDCHA(3,1)=10d-3
      ELSEIF (MAXBED.GE.6 .AND. MAXBED.LE.7) THEN
         NBED=4
         BEDCHA(2,1)=1d-3
         BEDCHA(3,1)=5d-3
         BEDCHA(4,1)=20d-3
      ELSEIF (MAXBED.GE.8) THEN
         NBED=5
         BEDCHA(2,1)=1d-3
         BEDCHA(3,1)=5d-3
         BEDCHA(4,1)=20d-3
         BEDCHA(5,1)=50d-3
      ELSE
         NBED=1  ! MAXBED must be larger than 2, therefore this should never be executed
      ENDIF

C Compute the density for each depth by numerical iterative aproximation
      ABOVE=0d0
      BEDCHA(1,3)=A-B-D
      BEDCHA(1,2)=TEROA*(BEDCHA(1,3)**TEROB)
      DO 20, IB=2,NBED
         N=0
         DZ=BEDCHA(IB,1)-BEDCHA(IB-1,1)
         M1=ABOVE+DZ*BEDCHA(IB-1,3)*0.95d0
         M2=ABOVE+DZ*BEDCHA(IB-1,3)*2d0
10       MMID=(M1+M2)/2d0
         IF(D.EQ.0) THEN
            BEDCHA(IB,3)=A-B*EXP(-C*MMID)
         ELSE
            BEDCHA(IB,3)=A-B*EXP(-C*MMID)-D*EXP(-E*MMID)
         ENDIF
         DZEST=(MMID-ABOVE)*2d0/(BEDCHA(IB,3)+BEDCHA(IB-1,3))
         n=n+1
         IF(ABS(DZ-DZEST).GT. 0.05d-3) THEN
            IF(DZEST.GT.DZ) THEN
               M2=MMID
            ELSE
               M1=MMID
            ENDIF
            GOTO 10
         ENDIF
         ABOVE=ABOVE+DZ*(BEDCHA(IB,3)+BEDCHA(IB-1,3))/2.
         BEDCHA(IB,2)=TEROA*(BEDCHA(IB,3)**TEROB)*
     &         (1+TEROC*(1-EXP(TEROD*ABOVE)))
20    CONTINUE

      DO 60, IB=NBED+1,MAXBED  ! set the rest of the table to 0
         BEDCHA(IB,1)=0d0
         BEDCHA(IB,2)=0d0
         BEDCHA(IB,3)=0d0
60    CONTINUE

      END   ! end of subroutine BEDINI

C ======================================================================
C ======================================================================
C SUBROUTINE REMOVEBED
C
C Merge one or several bed layers together be removing one or several
C rows in BEDCHA. The thickness of the bed layers is conserved. The density
C RHOdry is adjusted so to conserve the mass of sediment.
C If NB is an negative integer, this amount of rows are removed in BEDCHA.
C If NB is a positive real smaller than 0.5, then the layer thinner than
C this value (in metres) are merged.

      SUBROUTINE REMOVEBED(BEDCHA,MAXBED,NB)
      IMPLICIT NONE
C input variables
      INTEGER MAXBED         ! number of rows in variable BEDCHA
      DOUBLE PRECISION BEDCHA(MAXBED,3) ! bed characteristics in 3 column table
      DOUBLE PRECISION NB    ! parameter controlling the action of the subroutine
C intemediate variables
      INTEGER HIMAXBED
      PARAMETER (HIMAXBED=50) ! Highest possible value for MAXBED
      DOUBLE PRECISION DIFF(HIMAXBED) ! difference between interpolated
                              ! and old values for RHO
      INTEGER NBED            ! number of rows in BEDCHA that are used
      INTEGER N               ! number of rows in BEDCHA to be removed
                              ! if selection is on layer thickness, = 0

      DOUBLE PRECISION DZ     ! thickness of a layer
      DOUBLE PRECISION MINDZ  ! thickness of the thinnest layer / smallest difference
      INTEGER P               ! position of the row to be removed
      INTEGER I

C Find the number of rows that are used (NBED)
      DO 10, I=2,MAXBED
         IF (BEDCHA(I,1).EQ.0.0) GOTO 20 ! A zero depth indicate the last entry
10    CONTINUE
20    NBED=I-1
      IF(NBED.LT.3) RETURN    ! do nothing if there less than two layers

C Interpret the input argument NB
      IF(NB.LT.0) THEN
         N=MIN(-INT(NB-0.5d0),NBED-2)
            ! at least two entries must remains in BEDCHA
      ELSEIF(NB.EQ.0 .OR. NB.GT.0.5) THEN
         RETURN ! do nothing if NB is incorrect
      ELSE
         N=0
      ENDIF

C start of main loop (removing several rows in BEDCHA)
30    CONTINUE

C Compute how much RHOdry would change if interpolated
      DO 40, I=2,NBED-1
C for position z2, (dX-d1)/(z2-z1) = (d3-d1)/(z3-z2)
         DIFF(I)=(BEDCHA(I,1)-BEDCHA(I-1,1)) /
     &               (BEDCHA(I+1,1)-BEDCHA(I-1,1)) *
     &               (BEDCHA(I+1,3)-BEDCHA(I-1,3)) + BEDCHA(I-1,3)
     &               - BEDCHA(I,3)
40    CONTINUE

      IF(N.EQ.0) THEN
C CASE 1 : removal of the thinnest layer
C find the thinnest layer
         MINDZ=1000
         DO 50,I=NBED-1,1,-1
            DZ=BEDCHA(I+1,1)-BEDCHA(I,1)
            IF(DZ.LT.MINDZ) THEN
               P=I
               MINDZ=DZ
            ENDIF
50       CONTINUE
         IF(MINDZ.GE.NB) RETURN  ! quit if there is no (more) layer thinner than NB
C decide whether upper or limit of layer has to be removed
         IF(P.EQ.1 .OR.
     &   ( DIFF(P).GE.DIFF(P+1) .AND. P+1.LT.NBED ) ) THEN
            P=P+1
         ENDIF

      ELSE
C CASE 2 : removal of the best interpolated point
C find the best interpolated point
         P=NBED-1
         DO 60,I=NBED-2,2,-1
            IF(DIFF(I).LT.DIFF(P)) THEN
               P=I
            ENDIF
60       CONTINUE
         N=N-1
      ENDIF

C REMOVE THE ROW POSMIN
      IF(P.EQ.2) THEN
C particular case P=2, mass conservation for removing row 2
C (r1+r2)*(z2-0) + (r2+r3)*(z3-z2) = (X+r3)*(z3-0)  which can be simplified to
C X = r2 + (r1-r3)*z2/z3
         BEDCHA(1,3)=BEDCHA(2,3) + (BEDCHA(1,3)-BEDCHA(3,3))*
     &         BEDCHA(2,1)/BEDCHA(3,1)
      ELSE
C general case : mass conservation for removing row 3
C (r1+r2)*(z2-z1) + (r2+r3)*(z3-z2) + (r3+r4)*(z4-z3) =
C           (r1+X)*(z2-z1) + (X+r4)*(z4-z2)     which can be simplified to
C X = ( r2*(z3-z1) + r3*(z4-z2) + r4*(z2-z3) ) / (z4-z1)
         BEDCHA(P-1,3)=( BEDCHA(P-1,3)*(BEDCHA(P,1)-BEDCHA(P-2,1)) +
     &         BEDCHA(P,3)*(BEDCHA(P+1,1)-BEDCHA(P-1,1)) +
     &         BEDCHA(P+1,3)*(BEDCHA(P-1,1)-BEDCHA(P,1)) ) /
     &         (BEDCHA(P+1,1)-BEDCHA(P-2,1))
      ENDIF
C reshuffle the rows
      DO 70, I=P,NBED-1
         BEDCHA(I,1)=BEDCHA(I+1,1)
         BEDCHA(I,2)=BEDCHA(I+1,2)
         BEDCHA(I,3)=BEDCHA(I+1,3)
70    CONTINUE
      BEDCHA(NBED,1)=0d0
      NBED=NBED-1

      IF (NB.GT.0 .OR. N.GT.0) GOTO 30
C End of main loop (removing several rows in BEDCHA)

      END    ! end of subroutine REMOVEBED



C=======================================================================
C=======================================================================
C Subroutine SETCCONST
C Modify the default parameters of the common block MCONST by reading
C the values new values in the file FILENAME.
C INICONST must be called before any call to SETCCONST.
C A parameter remains unchanged if the new value is -999.
C No errror checks are done on the new values !
C If FILEMAME=' ', then write the example file INDATA.CST
C If at least one parameter is modified, then the modified parameter are
C written either
C  - in file opened with unit OUTUN (if OUTUN.ne.0)
C  - in file LOGFILE (if LOGFILE is not an empty string
C
      SUBROUTINE SETCCONST(FILENAME,OUTUN,LOGFILE)
      IMPLICIT NONE
      INCLUDE "sed_param.h"  ! declaration of the common blocks

      CHARACTER FILENAME*(*) ! Name of the *.CST file with the modified constants
      INTEGER OUTUN          ! File unit where the modification have to be documented
                             ! If = 0, then nothing is done
      CHARACTER LOGFILE*(*)  ! Name of a text file where the modifications are documented
                             ! If = ' ', then nothing is done

      LOGICAL EX
      CHARACTER NAME*30,CH*300
      INTEGER UN              ! file unit used for FILENAME
      INTEGER OUTUNIT         ! file unit for documenting the modifications
      INTEGER I
      LOGICAL LOG
      INTEGER LASTCH  ! Function
      DOUBLE PRECISION
     &XCSULVA,XTMULVA,XTRULVA,XRKERO,XE0,XCDISRUPT,XCLIM1,XCLIM2,
     &XKFLOC,XMFLOC,XRHOCLAY,XCTAUDEP,XPRS,XRHOMUD,XDPROFA,XDPROFB,
     &XDPROFC,XDPROFD,XDPROFE,XCONSOA,XTEROA,XTEROB,XTEROC,
     &XTEROD,XCDRAGRED,XZ0COH,XFCWCOH
      INTEGER XWSCLAY,XDOCOMPACT

      DO 10,UN=20,63
         INQUIRE(UNIT=UN,OPENED=EX)
         IF(.NOT.EX)GOTO 20
10    CONTINUE
      STOP
20    CONTINUE

      IF (OUTUN.NE.0)THEN
         OUTUNIT=OUTUN
      ELSEIF (LOGFILE.NE.' ') THEN
         DO 30,OUTUNIT=20,63
            INQUIRE(UNIT=OUTUNIT,OPENED=EX)
            IF(.NOT.EX)GOTO 40
30       CONTINUE
         STOP
40       CONTINUE
      ENDIF

      IF(FILENAME.EQ.' ')THEN
         NAME='INDATA.CST'
         INQUIRE(FILE=NAME, EXIST=EX)
         IF (EX) THEN
            WRITE(*,100) NAME(1:LASTCH(NAME))
            READ(*,*) CH
            IF (CH.NE.'Y' .AND. CH.NE.'y') THEN
               WRITE(*,101) NAME(1:LASTCH(NAME))
               STOP
            ENDIF
         ENDIF
         OPEN(UN,FILE=NAME,ERR=200)
         WRITE(UN,120,ERR=200)
         WRITE(CH,121)
         CALL LINETAB(CH)
         WRITE(UN,'(A)',ERR=200)CH
         WRITE(CH,*)(-999.,I=1,27),-999,-999
         CALL LINETAB(CH)
         WRITE(UN,'(A)',ERR=200)CH
         CLOSE(UN)
      ELSE
         OPEN(UN,FILE=FILENAME,ERR=210)
50       READ(UN,*,ERR=50,END=210)
     &   XCSULVA,XTMULVA,XTRULVA,XRKERO,XE0,XCDISRUPT,XCLIM1,XCLIM2,
     &   XKFLOC,XMFLOC,XRHOCLAY,XCTAUDEP,XPRS,XRHOMUD,XDPROFA,XDPROFB,
     &   XDPROFC,XDPROFD,XDPROFE,XCONSOA,XTEROA,XTEROB,XTEROC,
     &   XTEROD,XCDRAGRED,XZ0COH,XFCWCOH,XWSCLAY,XDOCOMPACT

         IF  (   ! Open LOGFILE and continue if any parameter is modified
     &   (XCSULVA.NE.-999.).OR.(XTMULVA.NE.-999.).OR.(XTRULVA.NE.-999.)
     &   .OR.(XRKERO.NE.-999.).OR.(XE0.NE.-999.).OR.(XCDISRUPT.NE.-999.)
     &   .OR.(XCLIM1.NE.-999.).OR.(XCLIM2.NE.-999.).OR.(XKFLOC.NE.-999.)
     &   .OR.(XMFLOC.NE.-999.).OR.(XRHOCLAY.NE.-999.).OR.(XCTAUDEP.NE.
     &    -999.).OR.(XPRS.NE.-999.).OR.(XRHOMUD.NE.-999.).OR.
     &   (XDPROFA.NE.-999.).OR.(XDPROFB.NE.-999.).OR.(XDPROFC.NE.-999.)
     &   .OR.(XDPROFD.NE.-999.).OR.(XDPROFE.NE.-999.).OR.(XCONSOA.NE.
     &   -999.).OR.(XTEROA.NE.-999.).OR.(XTEROB.NE.-999.).OR.(XTEROC.NE.
     &   -999.).OR.(XTEROD.NE.-999.).OR.(XCDRAGRED.NE.-999.).OR.(XZ0COH
     &   .NE.-999.).OR.(XFCWCOH.NE.-999.).OR.(XWSCLAY.NE.-999.).OR.
     &   (XDOCOMPACT.NE.-999.) ) THEN
     
            LOG = (OUTUN.NE.0) .OR. (LOGFILE.NE.' ')
     
            IF((OUTUN.EQ.0) .AND. (LOGFILE.NE.' '))
     &         OPEN(OUTUNIT,FILE=LOGFILE,ERR=220)

            IF(LOG)WRITE(OUTUNIT,140)

            IF(XCSULVA.NE.-999.)THEN
               CSULVA=XCSULVA
               IF(LOG)WRITE(OUTUNIT,141,ERR=220)'CSULVA',XCSULVA
            ENDIF
            IF(XTMULVA.NE.-999.)THEN
               TMULVA=XTMULVA
               IF(LOG)WRITE(OUTUNIT,141,ERR=220)'TMULVA',XTMULVA
            ENDIF
            IF(XTRULVA.NE.-999.)THEN
               TRULVA=XTRULVA
               IF(LOG)WRITE(OUTUNIT,141,ERR=220)'TRULVA',XTRULVA
            ENDIF
            IF(XRKERO.NE.-999.)THEN
               RKERO=XRKERO
               IF(LOG)WRITE(OUTUNIT,141,ERR=220)'RKERO',XRKERO
            ENDIF
            IF(XE0.NE.-999.)THEN
               E0=XE0
               IF(LOG)WRITE(OUTUNIT,141,ERR=220)'E0',XE0
            ENDIF
            IF(XCDISRUPT.NE.-999.)THEN
               CDISRUPT=XCDISRUPT
               IF(LOG)WRITE(OUTUNIT,141,ERR=220)'CDISRUPT',XCDISRUPT
            ENDIF
            IF(XCLIM1.NE.-999.)THEN
               CLIM1=XCLIM1
               IF(LOG)WRITE(OUTUNIT,141,ERR=220)'CLIM1',XCLIM1
            ENDIF
            IF(XCLIM2.NE.-999.)THEN
               CLIM2=XCLIM2
               IF(LOG)WRITE(OUTUNIT,141,ERR=220)'CLIM2',XCLIM2
            ENDIF
            IF(XKFLOC.NE.-999.)THEN
               KFLOC=XKFLOC
               IF(LOG)WRITE(OUTUNIT,141,ERR=220)'KFLOC',XKFLOC
            ENDIF
            IF(XMFLOC.NE.-999.)THEN
               MFLOC=XMFLOC
               IF(LOG)WRITE(OUTUNIT,141,ERR=220)'MFLOC',XMFLOC
            ENDIF
            IF(XRHOCLAY.NE.-999.)THEN
               RHOCLAY=XRHOCLAY
               IF(LOG)WRITE(OUTUNIT,141,ERR=220)'RHOCLAY',XRHOCLAY
            ENDIF
            IF(XCTAUDEP.NE.-999.)THEN
               CTAUDEP=XCTAUDEP
               IF(LOG)WRITE(OUTUNIT,141,ERR=220)'CTAUDEP',XCTAUDEP
            ENDIF
            IF(XPRS.NE.-999.)THEN
               PRS=XPRS
               IF(LOG)WRITE(OUTUNIT,141,ERR=220)'PRS',XPRS
            ENDIF
            IF(XRHOMUD.NE.-999.)THEN
               RHOMUD=XRHOMUD
               IF(LOG)WRITE(OUTUNIT,141,ERR=220)'RHOMUD',XRHOMUD
            ENDIF
            IF(XDPROFA.NE.-999.)THEN
               DPROFA=XDPROFA
               IF(LOG)WRITE(OUTUNIT,141,ERR=220)'DPROFA',XDPROFA
            ENDIF
            IF(XDPROFB.NE.-999.)THEN
               DPROFB=XDPROFB
               IF(LOG)WRITE(OUTUNIT,141,ERR=220)'DPROFB',XDPROFB
            ENDIF
            IF(XDPROFC.NE.-999.)THEN
               DPROFC=XDPROFC
               IF(LOG)WRITE(OUTUNIT,141,ERR=220)'DPROFC',XDPROFC
            ENDIF
            IF(XDPROFD.NE.-999.)THEN
               DPROFD=XDPROFD
               IF(LOG)WRITE(OUTUNIT,141,ERR=220)'DPROFD',XDPROFD
            ENDIF
            IF(XDPROFE.NE.-999.)THEN
               DPROFE=XDPROFE
               IF(LOG)WRITE(OUTUNIT,141,ERR=220)'DPROFE',XDPROFE
            ENDIF
            IF(XCONSOA.NE.-999.)THEN
               CONSOA=XCONSOA
               IF(LOG)WRITE(OUTUNIT,141,ERR=220)'CONSOA',XCONSOA
            ENDIF
            IF(XTEROA.NE.-999.)THEN
               TEROA=XTEROA
               IF(LOG)WRITE(OUTUNIT,141,ERR=220)'TEROA',XTEROA
            ENDIF
            IF(XTEROB.NE.-999.)THEN
               TEROB=XTEROB
               IF(LOG)WRITE(OUTUNIT,141,ERR=220)'TEROB',XTEROB
            ENDIF
            IF(XTEROC.NE.-999.)THEN
               TEROC=XTEROC
               IF(LOG)WRITE(OUTUNIT,141,ERR=220)'TEROC',XTEROC
            ENDIF
            IF(XTEROD.NE.-999.)THEN
               TEROD=XTEROD
               IF(LOG)WRITE(OUTUNIT,141,ERR=220)'TEROD',XTEROD
            ENDIF
            IF(XCDRAGRED.NE.-999.)THEN
               CDRAGRED=XCDRAGRED
               IF(LOG)WRITE(OUTUNIT,141,ERR=220)'CDRAGRED',XCDRAGRED
            ENDIF
            IF(XZ0COH.NE.-999.)THEN
               Z0COH=XZ0COH
               IF(LOG)WRITE(OUTUNIT,141,ERR=220)'Z0COH',XZ0COH
            ENDIF
            IF(XFCWCOH.NE.-999.)THEN
               FCWCOH=XFCWCOH
               IF(LOG)WRITE(OUTUNIT,141,ERR=220)'FCWCOH',XFCWCOH
            ENDIF
            IF(XWSCLAY.NE.-999)THEN
               WSCLAY=XWSCLAY
               IF(LOG)WRITE(OUTUNIT,142,ERR=220)'WSCLAY',XWSCLAY
            ENDIF
            IF(XDOCOMPACT.NE.-999)THEN
               DOCOMPACT=XDOCOMPACT
               IF(LOG)WRITE(OUTUNIT,142,ERR=220)'DOCOMPACT',XDOCOMPACT
            ENDIF

            IF(LOG)WRITE(OUTUNIT,'(/)',ERR=220)
            
            IF((OUTUN.EQ.0) .AND. (LOGFILE.NE.' '))
     &          CLOSE(OUTUNIT)


         ENDIF
         CLOSE(UN)
      ENDIF
      RETURN

200   WRITE(*,102) NAME(1:LASTCH(NAME))
      CLOSE (UN)
      STOP
210   WRITE(*,102) FILENAME(1:LASTCH(FILENAME))
      CLOSE (UN)
      STOP
220   WRITE(*,102) LOGFILE(1:LASTCH(FILENAME))
      CLOSE (UN)
      STOP

100   FORMAT (/'The existing file ',A,' will be overwritten, '/,
     &'do you want to continue ? (Y/N)')
101   FORMAT (/'Generation of file ',A,' aborted.')
102   FORMAT (/'Error during opening/writing file ',A,/)

120   FORMAT ('Example of input file for Sedtrans05 to modify the ',
     &'default values of the cohesive parameters.'/,
     &'Parameters with "-999" will keep their standard value.' )
121   FORMAT('CSULVA TMULVA TRULVA RKERO E0 CDISRUPT CLIM1 CLIM2 ',
     &'KFLOC MFLOC ',
     &'RHOCLAY CTAUDEP PRS RHOMUD DPROFA DPROFB DPROFC DPROFD DPROFE ',
     &'CONSOA TEROA TEROB TEROC TEROD CDRAGRED Z0COH FCWCOH ',
     &'WSCLAY DOCOMPACT')
140   FORMAT('Following default parameters for the cohesive algorithm',
     &' were modified')
141   FORMAT(T10,A,' was set to',G12.5)
142   FORMAT(T10,A,' was set to',I4)

      END    ! end of subroutine SETCCONST



C ======================================================================
C ======================================================================
C FUNCTION LASTCH
C
C Return the position of the last non space character of a string.
C Return 1 for an empty string
C
      INTEGER FUNCTION LASTCH(CH)
      CHARACTER CH*(*)
      INTEGER I
      DO 10, I=LEN(CH),2,-1
         IF (CH(I:I).NE.' ') GOTO 20
10    CONTINUE
20    LASTCH=I
      END



C ======================================================================
C ======================================================================
C  SUBROUTINE LINETAB
C
C Replace in a string all successive spaces with a tab
C
      SUBROUTINE LINETAB(CH)
      IMPLICIT NONE
      CHARACTER CH*(*),TAB
      INTEGER LENGTH,A,B
      INTEGER LASTCH   ! function
      LENGTH=LASTCH(CH);
      TAB=CHAR(9)
      A=0
      B=0

10    CONTINUE
         A=A+1
         B=B+1
         IF(CH(B:B).EQ.' ') THEN
            IF(B.NE.1) THEN
               CH(A:A)=TAB
            ELSE
               A=0
            ENDIF

20          CONTINUE
            IF(B.LT.LENGTH) THEN
               IF(CH(B+1:B+1).EQ.' ') THEN
                  B=B+1
                  GOTO 20
               ENDIF
            ENDIF

         ELSE
            CH(A:A)=CH(B:B)
         ENDIF
      IF(B.LT.LENGTH) GOTO 10

      IF(A.LT.LENGTH) CH(A+1:LENGTH)=' '
      END      ! end of subroutine LINETAB



C ======================================================================
C ======================================================================
C SUBROUTINE DENSVISC
C
C Compute the density and the dynamic visocity of water from the temperature
C and the salinity
      SUBROUTINE DENSVISC(TEMP,SALIN,RHOW,VISC)
      DOUBLE PRECISION TEMP   ! in degrees Celsius
      DOUBLE PRECISION SALIN  ! practical salinity scale (seawater=35)
      DOUBLE PRECISION RHOW   ! density of the (sea)water  (KG/M**3)
      DOUBLE PRECISION VISC   ! dynamic viscosity of the (sea)water (KG/M*SEC (OR N.S/M**2))

C compute the dynamic/molecular viscosity
      VISC=1.802863d-3 - 6.1086d-5*TEMP + 1.31419d-06*TEMP**2 -
     &1.35576d-08*TEMP**3 + 2.15123d-06*SALIN + 3.59406d-11*SALIN**2

C compute the water density according to Brydon et al. 1999, J. Geoph. Res.
C 104/C1, 1537-1540, equation 2 with Coefficient of Table 4, without pressure
C component. Ranges TEMP -2 - 40C, S 0-42, surface water.
C      RHOW=9.20601d-2 + 5.10768d-2*TEMP + 8.05999d-1*SALIN
C     &     -7.40849d-3*TEMP**2 - 3.01036d-3*SALIN*TEMP +
C     %     3.32267d-5*TEMP**3 + 3.21931d-5*SALIN*TEMP**2
C      RHOW=RHOW+1000d0

C compute the water density according to EOS80, Fofonoff 198599,
C J. Geoph. Res. 90/C2, 3332-3342, without pressure component.
      RHOW=999.842594d0 +6.793952d-2*TEMP -9.095290d-3*TEMP**2
     &   +1.00168d-4*TEMP**3 -1.120083d-6*TEMP**4 +6.536332d-9*TEMP**5
     & +(8.24493d-1 -4.0899d-3*TEMP +7.6438d-5*TEMP**2
     &   -8.2467d-7*TEMP**3 +5.3875d-9*TEMP**4) * SALIN
     & +(-5.72466d-3 +1.0227d-4*TEMP -1.6546d-6*TEMP**2) * SALIN**1.5d0
     & +4.8314d-4*SALIN**2

      END       ! end of subroutine DENSVISC



C ======================================================================
C ======================================================================
C SUBROUTINE INICONST
C
C Initialize the constants in the common block CCONST.
C Must be called before the execution of the COHESIVE subroutine, except
C if ALL constants are assigned separately.
C
      SUBROUTINE INICONST
      IMPLICIT NONE

      INCLUDE "sed_param.h"  ! declaration of the common blocks

C intermediate variable to construct WSI and DISTR
      INTEGER I
      DOUBLE PRECISION WSTART    ! slowest Ws
      DOUBLE PRECISION DWSLOG    ! LOG10 of ratio of two successive Ws
      DOUBLE PRECISION STDEV     ! standard deviation of log10 normal distribution
      DOUBLE PRECISION A,MEAN,SUMNORM

C Constants for solid transmitted stress by Ulva
      CSULVA=159.4d0   ! coefficient for the solid stransmitted stress by Ulva
      TMULVA=1.054d-3  ! threshold of motion of Ulva (Pa)
      TRULVA=0.0013d0  ! threshold of full resuspension of Ulva (Pa)
C Constants for erosion formula
      E0=1.95d-5      ! customized values for V20 (Sea Carousel Venice, winter 99)
      RKERO=5.88d0
C Constant for the sediment put in suspension
      WSCLAY=5 ! WSCLAY must NOT be smaller than NDISTR
      ! constant for turbulent disruption
C      CDISRUPT=0.001d0  ! calibrated for Sea Carousel Venice winter 99
      CDISRUPT=0.001d0
C Constants for the flocculation
      CLIM1=0.1d0   !
      CLIM2=2d0     !
      KFLOC=0.001d0 ! default 0.001
      MFLOC=1d0     ! default 1
      RHOCLAY=2600d0
C Constant for deposition formula
      CTAUDEP=1d0   ! ideally 1
      CTAUDEP=10d0   ! ideally 1
      PRS=0d0   ! range 0-1
C Constants for freshly deposited sediment
      RHOMUD=50d0
C Constants for density profile
C     FINAL(I) = A -  B * EXP(-C*ABOVE(I)) - D * EXP(-E*ABOVE(I))
C     A : final deep density
C     A-B-D : final surface density
C     C and E : define the shape (in conjunction with B and C)
      DPROFA=470d0
      DPROFB=150d0
      DPROFC=0.015d0
      DPROFD=0d0
      DPROFE=0d0
C Constant for consolidation
      CONSOA=1d-5
C Constants for erosion threshold from density and overlaying mass
      TEROA=6d-10
      TEROB=3d0
      TEROC=3.47d0
      TEROD=-1.915d0

C Ws of each Ws class, depending of number of Ws classes (NBCONC) : WSI
      WSTART=1d-5
      IF(NBCONC.GT.33 .OR. NBCONC.LT.5)THEN
         WRITE(*,*)'Incorrect value of NBCONC'
         STOP
      ELSEIF(NBCONC.GE.25) THEN
         DWSLOG=0.125d0
      ELSEIF(NBCONC.GE.22) THEN
         DWSLOG=1d0/6d0
      ELSEIF(NBCONC.GE.15) THEN
         DWSLOG=0.2d0
         IF (NBCONC.EQ.15) WSTART=10d0**(-4.8d0)
      ELSEIF(NBCONC.GE.11) THEN
         DWSLOG=0.25d0
         IF (NBCONC.LE.12) WSTART=10d0**(-4.75d0)
      ELSEIF(NBCONC.GE.8) THEN
         DWSLOG=LOG10(2d0)
         IF (NBCONC.GT.8) THEN
            WSTART=0.00002d0
         ELSE
            WSTART=0.00004d0
         ENDIF
      ELSEIF(NBCONC.EQ.7) THEN
         DWSLOG=LOG10(2.5d0)
         WSTART=0.00002d0
      ELSE
         DWSLOG=LOG10(3d0)
         IF (NBCONC.GT.5) THEN
            WSTART=0.00002d0
         ELSE
            WSTART=0.00005d0
         ENDIF
      ENDIF
      DO 100,I=1,NBCONC
100      WSI(I)=WSTART * 10d0**(DBLE(I-1)*DWSLOG)

C Shape of Ws distribution put in suspension : DISTR
C compute normal distribution and scale it so that sum(DISTR)=1
C   normal = 1/(STDEV*sqrt(2*PI) * exp( -0.5*((x-MEAN)/STDEV)**2)
      STDEV=0.3d0  ! standard deviation of LOG10(Ws)
      NDISTR=MIN(2*INT(0.7/DWSLOG+0.3)+1,15) ! number of used elements in DISTR
      MEDDISTR=(NDISTR+1)/2
      MEAN=MEDDISTR*DWSLOG
      A=1/(STDEV*SQRT(2d0*2d0*ACOS(0d0)))
      SUMNORM=0d0
      DO 210,I=1,NDISTR
         DISTR(I)=A*EXP(-0.5d0*((I*DWSLOG-MEAN)/STDEV)**2) * DWSLOG
         SUMNORM=SUMNORM+DISTR(I)
210   CONTINUE
      DO 220,I=1,NDISTR
         DISTR(I)=DISTR(I)/SUMNORM
220   CONTINUE

C Switch for calling performing compaction at each call to COHESIVE
      DOCOMPACT=0     ! <> 0 call COMPACT, = 0 do not call compact
C Constant for the drag reduction formula : TAU0effectif=TAU0*exp(CDRAGRED*SSC)
      CDRAGRED=-0.0893d0   ! CDRAGRED must be lower or equal to 0
                           ! set CDRADREG=0 for no drag reduction
C ======================================================================
C                         MAIN
C ======================================================================

      G = 9.81
      PII = 2.*ASIN(1d0)

C ======================================================================
C                         FRICFAC
C ======================================================================

      Z0COH = 2.0D-4
      FCWCOH = 2.2D-3

C ----------------------------------------------------------------
      END    ! end of subroutine INICONST
